%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global controller desgin
%  NOTE: Call after linearization_all
%
% Gen:
% dot{x} = sys.Ag*x + sys.Rg*x2 + sys.B*u,  x2 is state of renewable
% y         = sys.C*x
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch 1
  case 1
    % **************************************
    % Standard observer-based control
    % dot{xhat} = gl.K*xhat + gl.H*y
    % u = gl.F*xhat
    % **************************************
    Q = sys.C'*sys.C;
    gl.H = lqr(sys.Ag', sys.C', 100*eye(net.ng), eye(size(sys.C,1)))'; 
    gl.F = -lqr(sys.Ag, sys.B, 1e4*Q, eye(NG)); 
    gl.K = sys.Ag+sys.B*gl.F-gl.H*sys.C; 
    tmpn = n + n_dae(ones(1,NW), ones(1,NP));
    gl.x0 = zeros(net.ng,1); 

    %sys.B = [sys.B; zeros(7, NG)]; 
    %sys.C = EYE(net.n, out.ome)';
    %gl.H = lqr(sys.A', sys.C', 1e3*eye(net.n), eye(size(sys.C,1)))'; 
    %gl.F = -lqr(sys.A, sys.B, 1e4*Q, eye(NG)); 
    %gl.K = sys.A+sys.B*gl.F-gl.H*sys.C; 
    %tmpn = n + n_dae(ones(1,NW), ones(1,NP));
    %gl.x0 = zeros(net.n,1); 
    gl.n = size(gl.K, 1); 
    gl.si = [tmpn+1:tmpn+gl.n];

    
    % error check
    flag = 1*ones(1,NW); %with compensator
    flag_pv = 1*ones(1,NP); %with compensator
    flag_g = 1;
    ff = 0;
    n_ode = n + n_dae(flag, flag_pv) + gl.n; 
    dx = psm_15s(0, [init_x;init_w;gl.x0]);
    if(norm(dx) > 1e-5); error('failed determining initial global controller state '); end

end

if 1
    % step size
  h = 1e-7; 
  
  % linearized A
  x0 = [init_x;gl.x0]; 

  tmpA = zeros(length(x0)); 
  flag = -1*ones(1,NW);
  flag_g = 1;
  flag_pv = -1*ones(1,NP); 
  ff = 0; 
  n_ode = n + n_dae(flag, flag_pv); 
  tmpgl = gl.si;
  gl.si = n + [1:gl.n]; %tentatively redefine state indice
  for k=1:length(x0)
    e = h*EYE(length(x0), k); 
    tmpA(:,k) = (psm_15s(0, x0+e) - psm_15s(0, x0))/h; 
  end
  gl.si = tmpgl; %recover state indice
  
  %partion of A
  A11 = tmpA(1:net.n, 1:net.n); 
  A12 = tmpA(1:net.n, net.n+1:net.n+net.nDAE); 
  A13 = tmpA(1:net.n, net.n+net.nDAE+1:end);
  A21 = tmpA(net.n+1:net.n+net.nDAE, 1:net.n); 
  A22 = tmpA(net.n+1:net.n+net.nDAE, net.n+1:net.n+net.nDAE); 
  A23 = tmpA(net.n+1:net.n+net.nDAE, net.n+net.nDAE+1:end);
  A31 = tmpA(net.n+net.nDAE+1:end, 1:net.n); 
  A32 = tmpA(net.n+net.nDAE+1:end, net.n+1:net.n+net.nDAE); 
  A33 = tmpA(net.n+net.nDAE+1:end, net.n+net.nDAE+1:end);
  
  %kron-reduction
  whole_g.A = [A11,A13;A31,A33] - [A12;A32]*inv(A22)*[A21,A23];

  %eigenvalues
  [whole_g.vec, E] = eig(whole_g.A); whole_g.E = diag(E); 
  
  % eigA .... eigenvalues of system without any controllers
  % whole_g.E .... eigenvalues of system with global

  figure; plot(real(whole_g.E), imag(whole_g.E), 'bo'); hold on; plot(real(eigA), imag(eigA), 'rs'); grid on; ylim([-2 2]); 
  pause
end