%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Created by Tomonoi Sadamoto.
% Updated at 11, Sep. 2018
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; pause(0.1); addpath('./tool/');
global net NG NL NW NP gpar dae Vref vfdstar win flag mpc pm sims
global compw lin delPW ystar init_PL init_EL init_PP init_IP YL x0_dae Yk n_ode ff Ykc init_EW 
global net_f fcase Yf Yf_wof init_x
global ir_s is_s ws vdc_s Q_s
global Yg Yg_iw Yg_ipv wr_s
global P_spv Q_spv ipv_s mG_s pv flag_pv Spv_s
global compw_pv
global flag_g addPSSu gl sys 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Select a power network to be simulated
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% **** penetration level ****
gamma = [60]; %wind farm
gamma_pv = [200]; %pv farm (200)

% **** load wind & storage ****
data_wind01; 
data_pv01; 

% **** parameters of retrofit controller ****
gains = 100;
gain_i = [1];
nhat = [16];
nhat_pv = [7];

% fault bus
sims.fbus = 10;

% **** load other settings ****
data_68bus; 

if mpc.wind_model == 0
  nhat = [9];
elseif mpc.wind_model == 1 & sto_flag == 1
  nhat = 18;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Power flow calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pflow;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pre-calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
precalc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initial settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
init_state; 

% error check
flag = -1*ones(1,NW); %-1: use traditional controller
flag_pv = -1*ones(1,NP); %-1: use traditional controller
flag_g = -1; %without use global controller
ff = 0; %0: power line fault flag off
n_ode = n; %dimension of whole system
delPW = [0;0]; %must be 0
x0_dae = init_x; %equilibrium
dx = psm_15s(0, init_x);
if(norm(dx) > 1e-5); error('failed determining initial state'); end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setup 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sims.x0type = -1;
sims.type = 1; %1-nonlinear sim   0-linear sim 

%make disturbance of initial state
if(sims.x0type == 1)
  ds = zeros(n,1);
  for j=1:NW
    ds(win{j}.si(4:7)) = -0.01 + 0.02*rand(4,1);
  end
  save('ds_wind01.mat','ds');
  %ds(out.del(1:2)) = pi/180*(-1+ 1*rand(length(1:2),1));
elseif(sims.x0type == 2)
  ds = zeros(n,1);
  for j=1:NP
    %ds(pv{j}.si(1:2)) = -0.01+ 0.02*rand(2,1);
    ds(pv{j}.si(1:2)) = -0.7*init_x(pv{j}.si(1:2)); 
  end
  save('ds_pv01.mat','ds'); 
  %ds(out.del(1:2)) = pi/180*(-1+ 1*rand(length(1:2),1));
elseif(sims.x0type == 3)
  ds = zeros(n,1);
  for j=1:NP
    ds(pv{j}.si(7)) = -0.3*init_x(pv{j}.si(7)); 
  end
  save('ds_pv01.mat','ds');
elseif(sims.x0type == 4)
  ds = zeros(n,1);
  ds(win{1}.si(4:5)) = -0.1*init_x(win{1}.si(4:5)); 
  %ds(win{1}.si(6:7)) = -0.2*init_x(win{1}.si(6:7)); 
elseif(sims.x0type == 5)
  ds = zeros(n,1);
  ds(win{1}.si(1)) = -0.7*init_x(win{1}.si(1)); 
  %ds(win{1}.si(6:7)) = -0.2*init_x(win{1}.si(6:7)); 
elseif(sims.x0type == 99)
  %load('ds_pv01.mat'); 
  load('ds_wind01.mat');
  tmp = ds(win{1}.si(4:7)); ds = zeros(n,1); ds(win{1}.si(4:7)) = tmp;
  
elseif(sims.x0type == -1)
  % ******************** make fault settings ************************
  ds = zeros(n,1); 
  fprintf('[ Fault Simulation (Bus = %d)]\n', sims.fbus);
  
  % do settings
  make_fault_settings; 
  
  % ***** determine initial disturbance *****
  % Note:  state = genstate; EL; EW; EP and ELF
  n_ode = n; 
  flag = -1*ones(1,NW); %traditional controller
  flag_pv = -1*ones(1,NP); %-1: use traditional controller
  flag_g = -1; %without use global controller
  x0_dae = init_x; %equilibrium
  ff = 1; %fault flag on
  options = odeset('Mass',Mf,'RelTol',1e-8,'AbsTol',1e-8);
  [tf,xf] = ode15s(@psm_15s,[-sims.ft:0.001:0], init_xf, options);sims.odemem = 0;
  ds = xf(end, 1:n)'-init_x(1:n); 
  tf = tf(1:end-1); xf = xf(1:end-1, :); 
  ff = 0; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Linearization 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
linearization_all;

for j=1:NW
  linearization; 
end
for j=1:NP
  linearization_pv; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation without any control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sims.type == 1
if sims.type
  fprintf('====== Nonlinear simulation ======\n'); 
  flag = -1*ones(1,NW); %traditional controller
  flag_pv = -1*ones(1,NP); %-1: use traditional controller
  flag_g = -1; %without use global controller
  x0_dae = init_x; %equilibrium
  n_ode = n; 
  
  options = odeset('Mass',Mwo,'RelTol', tol,'AbsTol', tol); 
  [to,xo] = ode15s(@psm_15s,[0:0.01:sims.te], init_x+ds, options);sims.odemem = 0;

  if(sims.x0type == -1); to = [tf; to]; xo = [xf(:,1:n); xo]; end
  % add pre fault trajectories for journal
  [to,xo] = add_prefault_traj(to,xo,-1, init_x); 
else
  fprintf(' ====== Linear simulation ======\n'); 
  options = odeset('RelTol',1e-8,'AbsTol',1e-8);
  [to,xo] = ode45(@(t,x) sys.A*x, [0:0.01:sims.te], ds(1:net.n), options);
  xo = xo + ones(length(to),1)*init_x(1:net.n)'; 
  % add pre fault trajectories for journal
  [to,xo] = add_prefault_traj(to,xo,-1, init_x(1:net.n)); 
end

% calculate some physical quantities
t=to; x=xo;
cal_sg; if(NW>0); cal_wind; end; if(NP>0&mpc.PVtype==3); cal_pv; end; 
%plot if you want 
pl01;if(NW>0); pl02; end; if(NP>0&mpc.PVtype==3); pl03; end; 

% save
res_ori.t = to;
res_ori.ome = ome;
res_ori.thdiff = thdiff;
res_ori.vG = vabs_G; 
res_ori.vL = vabs_L; 
if NP>0
  res_ori.ig_pv = ig_pv; 
  res_ori.vdc_pv = vdc_pv; 
  res_ori.mG = mGpv;
  res_ori.PP = PP;
end
if NW>0
  res_ori.wl = wl;
  res_ori.wr = wr;
  res_ori.thT = thT;
  res_ori.is = is;
  res_ori.ir = ir;
  res_ori.P = PW;
  if(mpc.wind_model==1)
    res_ori.ig = ig;
    res_ori.vdc = vdc;
    res_ori.mG = mG;
    if sto_flag==1
      res_ori.vb = vb;
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Retrofit Controller Design 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for lp=1:length(gain_i)
  % ***** Hierarchical Controller Design *****
  gain = gains(gain_i(lp));
  design_compensators; 
  result{lp}.E = wholew.E;
  result{lp}.vec = wholew.vec;  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global Controller Design 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%des_global;
global_eig; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation 
% change simulation type depending on your purpose
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sim_singleRetrofit; 
sim_allRetrofit; 
sim_Global; 
sim_allRetrofit_Global; 


%pl_art01; %Plot (single retrofit on wind)
%pl_art02; %Plot (compare single retrofit/multiple retrofit)
pl_art03; %Plot (compare no/all retrofit/global/ all retrfoti+global )