% ********************************************
% low-dimensional system
% ********************************************
function [A, B, G, P, Pdag, varargout] = make_lowdim(Ain, Bin, nhat)
global meth

if nhat == size(Ain,1)
  %%%%%%%%%%%%%%%%%%%%%%%%%%
  % Use full-dimensional model
  %%%%%%%%%%%%%%%%%%%%%%%%%%
  A = Ain;
  B = Bin;
  G = 0;
  P = eye(nhat);
  Pdag = eye(nhat);
  varargout{1} = []; 
else
  %%%%%%%%%%%%%%%%%%%%%%%%%%
  % Use low-dimensional model
  %%%%%%%%%%%%%%%%%%%%%%%%%%  
  % ===== Find P making || G - Ghat||_Hinf small =====
  if meth(j) == 1
    lin{j}.sigma = inf;
  else
    lin{j}.sigma = 0;
  end
  
  % Balanced realization
  [sysb,hs,T,Ti] = balreal(ss(Alin, Blin, eye(size(Alin,1))', []));
  W = EYE(lin{j}.n, [1:nhat(j)-lin{j}.m]);
  
  % Make WQ including basis of B or [B x0]
  [Qq,R] = qr(null(W')*null(W')'*T*[Blin]); 
  WQ = [W,Qq(:,1:rank(R))];
  %WQ = W;
  
  %Define projection matrices
  lin{j}.P = Ti*WQ;
  lin{j}.Pd = WQ'*T;
  lin{j}.Pb = Ti*null(WQ');
  lin{j}.Pbd= null(WQ')'*T;

  %Check orthogonality
  if nhat(j) < lin{j}.n
    if(norm(lin{j}.Pbd*lin{j}.P) > 1e-8)
      error('P  Pbd ĂȂ');
    elseif(norm(lin{j}.Pd*lin{j}.Pb) > 1e-8)
      error('Pb  Pd ĂȂ');
    elseif(norm(lin{j}.P*lin{j}.Pd + lin{j}.Pb*lin{j}.Pbd - eye(lin{j}.n)) > 1e-6)
      error('P*Pd + Pb*Pbd = I藧Ȃ');  
    end
    if(norm(lin{j}.Pbd*Blin) > 1e-3)
      error('B  P Ɋ܂܂ĂȂ');
    end
  end

  % ===== Define low-dim system ======
  lin{j}.nhat = size(lin{j}.Pd, 1); 
  
  if(lin{j}.sigma == inf & lin{j}.nhat ~= lin{j}.n)
    PI = 0;
    lin{j}.Gam = lin{j}.Pb*lin{j}.Pbd;
    lin{j}.G = lin{j}.Pd*Alin*lin{j}.Gam;
  elseif(lin{j}.nhat ~= lin{j}.n)
    PI = lin{j}.Pb*inv(lin{j}.sigma*eye(lin{j}.n - lin{j}.nhat) - lin{j}.Pbd*Alin*lin{j}.Pb)*lin{j}.Pbd;  
    lin{j}.Gam = lin{j}.sigma*PI - PI*Alin;
    lin{j}.G = lin{j}.Pd*Alin*lin{j}.Gam;
  else
    lin{j}.G = 0; PI = 0;
  end
  Alinhat = lin{j}.Pd*Alin*lin{j}.P + lin{j}.Pd*Alin*PI*Alin*lin{j}.P; 
  Blinhat = lin{j}.Pd*Blin + lin{j}.Pd*Alin*PI*Blin; 
  lin{j}.Powhat = lin{j}.Pow*(lin{j}.P + PI*Alin*lin{j}.P);

  fprintf('%d -> %d\n', lin{j}.n, lin{j}.nhat);

end
