%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot Details of Wind System States
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
col = [0 0 1; 1 0 0; 0 0.8 0; 1 1 0; 0.5 0.5 0.1]; 
figure;
for j=1:NW
  % ***********************************
  % Rotor angular speed (Hz)
  % ***********************************
  subplot(2,3,1);
  plot(t, wr(:,j),'linewidth',1); hold on; 
  plot(t(end), (1/(2*pi))*(omega0/2)*wr_s{j},'bo','linewidth',1);
  grid on; ylabel('wr[Hz]'); xlim([t(1) t(end)]); 
  
  % ***********************************
  % DFIG current (pu)
  % ***********************************
  subplot(2,3,2);
  plot(t, is(:,2*j-1),'b-','linewidth',1); hold on; 
  plot(t, is(:,2*j),'r-','linewidth',1); 
  plot(t, ir(:,2*j-1),'b--','linewidth',1); 
  plot(t, ir(:,2*j),'r--','linewidth',1); 
  plot(t(end), is_s{j}(1),'bo','linewidth',1);
  plot(t(end), is_s{j}(2),'ro','linewidth',1);
  plot(t(end), ir_s{j}(1),'bs','linewidth',1);
  plot(t(end), ir_s{j}(2),'rs','linewidth',1);
  grid on; ylabel('DFIG current[pu]'); xlim([t(1) t(end)]); 
  legend('ids', 'iqs'); 
  
  % ***********************************
  % Active and Reactive power to grid (MW)
  % ***********************************
  subplot(2,3,3);
  plot(t, mpc.baseMVA*[real(PW(:, j)), imag(PW(:, j))],'linewidth',1); hold on; 
  plot(t(end), mpc.baseMVA*init_PW(1),'bo','linewidth',1); 
  plot(t(end), mpc.baseMVA*init_PW(2),'ro','linewidth',1); 
  grid on; ylabel('Pow to Grid(MW)'); xlim([t(1) t(end)]);  legend('P', 'Q'); 

  if(mpc.wind_model==1)
    % ***********************************
    % Duty Cycle 
    % ***********************************  
    subplot(2,3,4);
    plot(t, mG{j}(:, 1),'b-','linewidth',1); hold on; 
    plot(t, mG{j}(:, 2),'r-','linewidth',1);
    plot(t, mR{j}(:, 1),'b--','linewidth',1); hold on; 
    plot(t, mR{j}(:, 2),'r--','linewidth',1);
    plot([t(1),t(end)], [1,1],'k-','linewidth',0.5);
    plot([t(1),t(end)], [-1,-1],'k-','linewidth',0.5);
    grid on; ylabel('Duty'); xlim([t(1) t(end)]);  legend('mdG', 'mqG'); 
  
    subplot(2,3,5);
    plot(t, vdc(:, j),'linewidth',1);  hold on;
    grid on; ylabel('Vdc [V]'); xlim([t(1) t(end)]); 
    
    subplot(2,3,6);
    plot(t, mpc.baseMVA*[Powg{j}(:, 1), Powg{j}(:, 2)],'linewidth',1); hold on; 
    plot(t(end), gamma*mpc.baseMVA*winp(j,10),'bo','linewidth',1); 
    plot(t(end), gamma*mpc.baseMVA*winp(j,11),'ro','linewidth',1); 
    grid on; ylabel('Pow to GSC(MW)'); xlim([t(1) t(end)]);  legend('P', 'Q'); 
  end
end
 
if 1 & mpc.wind_model
  figure; 
  for j=1:NW
    % ***********************************
    % iG
    % ***********************************
    subplot(2,3,1);
    plot(t, ig(:,2*j-1),'b-','linewidth',1); hold on; 
    plot(t, ig(:,2*j),'r-','linewidth',1); hold on; 
    grid on; ylabel('iG'); xlim([t(1) t(end)]); legend('d','q');
  
    if sto_flag == 1
      subplot(2,3,2);
      plot(t, vb,'b-','linewidth',1); hold on; 
      grid on; ylabel('vb(V)'); xlim([t(1) t(end)]); 
      
      subplot(2,3,3);
      plot(t, idcP,'b-','linewidth',1); hold on; 
      grid on; ylabel('idcP(pu)'); xlim([t(1) t(end)]); 
      
      subplot(2,3,4);
      plot(t, Sdc,'b-','linewidth',1); hold on; 
      plot([t(1),t(end)], [0,0],'k:','linewidth',0.5); 
      grid on; ylabel('Duty of DC/DC'); xlim([t(1) t(end)]); 
    end
  end
end